/* A Bison parser, made by GNU Bison 2.5.  */

/* Bison implementation for Yacc-like parsers in C
   
      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.
   
   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output.  */
#define YYBISON 1

/* Bison version.  */
#define YYBISON_VERSION "2.5"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 0

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1

/* Using locations.  */
#define YYLSP_NEEDED 0



/* Copy the first part of user declarations.  */

/* Line 268 of yacc.c  */
#line 1 "fpd.y"

/*
 * fpd.y - FootPrint Definition language
 *
 * Written 2009-2011 by Werner Almesberger
 * Copyright 2009-2011 by Werner Almesberger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */


#include <stdlib.h>
#include <string.h>

#include "util.h"
#include "error.h"
#include "coord.h"
#include "expr.h"
#include "obj.h"
#include "meas.h"
#include "gui_status.h"
#include "gui_inst.h" /* for %meas */
#include "dump.h"
#include "tsort.h"
#include "fpd.h"

#include "y.tab.h"


struct expr *expr_result;
const char *var_id;
struct value *var_value_list;


static struct frame *curr_frame;
static struct table *curr_table;
static struct row *curr_row;

static struct vec *last_vec = NULL;

static struct table **next_table;
static struct loop **next_loop;
static struct vec **next_vec;
static struct obj **next_obj;

static int n_vars, n_values;

static const char *id_sin, *id_cos, *id_sqrt;

static struct tsort *tsort;


/* ----- lookup functions -------------------------------------------------- */


static struct frame *find_frame(const char *name)
{
	struct frame *f;

	for (f = frames->next; f; f = f->next)
		if (f->name == name)
			return f;
	return NULL;
}


static struct vec *find_vec(const struct frame *frame, const char *name)
{
	struct vec *v;

	for (v = frame->vecs; v; v = v->next)
		if (v->name == name)
			return v;
	return NULL;
}


static struct obj *find_obj(const struct frame *frame, const char *name)
{
	struct obj *obj;

	for (obj = frame->objs; obj; obj = obj->next)
		if (obj->name == name)
			return obj;
	return NULL;
}


static int find_label(const struct frame *frame, const char *name)
{
	if (find_vec(frame, name))
		return 1;
	if (find_obj(frame, name))
		return 1;
	return 0;
}


static struct var *find_var(const struct frame *frame, const char *name)
{
	const struct table *table;
	struct var *var;
	struct loop *loop;

	for (table = frame->tables; table; table = table->next)
		for (var = table->vars; var; var = var->next)
			if (var->name == name)
				return var;
	for (loop = frame->loops; loop; loop = loop->next)
		if (loop->var.name == name)
			return &loop->var;
	return NULL;
}


/* ----- item creation ----------------------------------------------------- */


static void set_frame(struct frame *frame)
{
	curr_frame = frame;
	next_table = &frame->tables;
	next_loop = &frame->loops;
	next_vec = &frame->vecs;
	next_obj = &frame->objs;
	last_vec = NULL;
}


static void make_var(const char *id, struct expr *expr)
{
	struct table *table;

	table = zalloc_type(struct table);
	table->vars = zalloc_type(struct var);
	table->vars->name = id;
	table->vars->frame = curr_frame;
	table->vars->table = table;
	table->rows = zalloc_type(struct row);
	table->rows->table = table;
	table->rows->values = zalloc_type(struct value);
	table->rows->values->expr = expr;
	table->rows->values->row = table->rows;
	table->active_row = table->rows;
	*next_table = table;
	next_table = &table->next;
}


static void make_loop(const char *id, struct expr *from, struct expr *to)
{
	struct loop *loop;

	loop = alloc_type(struct loop);
	loop->var.name = id;
	loop->var.next = NULL;
	loop->var.frame = curr_frame;
	loop->var.table = NULL;
	loop->from.expr = from;
	loop->from.row = NULL;
	loop->from.next = NULL;
	loop->to.expr = to;
	loop->to.row = NULL;
	loop->to.next = NULL;
	loop->next = NULL;
	loop->active = 0;
	loop->initialized = 0;
	*next_loop = loop;
	next_loop = &loop->next;
}


static struct obj *new_obj(enum obj_type type)
{
	struct obj *obj;

	obj = alloc_type(struct obj);
	obj->type = type;
	obj->name = NULL;
	obj->frame = curr_frame;
	obj->next = NULL;
	obj->lineno = lineno;
	return obj;
}


/* ---- frame qualifiers --------------------------------------------------- */


static int duplicate_qualifier(const struct frame_qual *a,
    const struct frame_qual *b)
{
	if (!b)
		return 0;
	if (a != b && a->frame == b->frame) {
		yyerrorf("duplicate qualifier \"%s\"", a->frame->name);
		return 1;
	}
	if (b && duplicate_qualifier(a, b->next))
		return 1;
	return duplicate_qualifier(a->next, a->next);
}


static int can_reach(const struct frame *curr, const struct frame_qual *qual,
    const struct frame *end)
{
	const struct obj *obj;

	if (curr == end)
		return !qual;

	/*
	 * Don't recurse for removing the qualifier alone. We require a frame
	 * reference step as well, so that things like foo.foo.foo.bar.vec
	 * aren't allowed.
	 *
	 * Since a duplicate qualifier can never work, we test for this error
	 * explicitly in "duplicate_qualifier".
	 */
	if (qual && curr == qual->frame)
		qual = qual->next;

	for (obj = curr->objs; obj; obj = obj->next)
		if (obj->type == ot_frame)
			if (can_reach(obj->u.frame.ref, qual, end))
				return 1;
	return 0;
}


static int check_qbase(struct qbase *qbase)
{
	if (duplicate_qualifier(qbase->qualifiers, qbase->qualifiers))
		return 0;

	if (!can_reach(frames, qbase->qualifiers, qbase->vec->frame))
		yywarn("not all qualifiers can be reached");
	return 1;
}


/* ----- debugging directives ---------------------------------------------- */


static int dbg_delete(const char *frame_name, const char *name)
{
	struct vec *vec;
	struct obj *obj;
	struct frame *frame;

	if (!frame_name)
		frame = curr_frame;
	else {
		frame = find_frame(frame_name);
		if (!frame) {
			yyerrorf("unknown frame \"%s\"", frame_name);
			return 0;
		}
	}
	vec = find_vec(frame, name);
	if (vec) {
		delete_vec(vec);
		return 1;
	}
	obj = find_obj(frame, name);
	if (obj) {
		delete_obj(obj);
		return 1;
	}
	if (!frame_name) {
		frame = find_frame(name);
		if (frame) {
			if (curr_frame == frame) {
				yyerrorf("a frame can't delete itself");
				return 0;
			}
			delete_frame(frame);
			return 1;
		}
	}
	if (frame_name)
		yyerrorf("unknown item \"%s.%s\"", frame_name, name);
	else
		yyerrorf("unknown item \"%s\"", name);
	return 0;
}


static int dbg_move(const char *name, int anchor, const char *dest)
{
	struct vec *to, *vec;
	struct obj *obj;
	struct vec **anchors[3];
	int n_anchors;

	to = find_vec(curr_frame, dest);
	if (!to) {
		yyerrorf("unknown vector \"%s\"", dest);
		return 0;
	}
	vec = find_vec(curr_frame, name);
	if (vec) {
		if (anchor) {
			yyerrorf("invalid anchor (%d > 0)", anchor);
			return 0;
		}
		vec->base = to;
		return 1;
	}
	obj = find_obj(curr_frame, name);
	if (!obj) {
		yyerrorf("unknown item \"%s\"", name);
		return 0;
	}
	n_anchors = obj_anchors(obj, anchors);
	if (anchor >= n_anchors) {
		yyerrorf("invalid anchor (%d > %d)", anchor, n_anchors-1);
		return 0;
	}
	*anchors[anchor] = to;
	return 1;
}


/*
 * @@@ This is very similar to what we do in rule "obj". Consider merging.
 */

/*
 * We need to pass base_frame and base_vec, not just the vector (with the
 * frame implied) since we can also reference the frame's origin, whose
 * "vector" is NULL.
 */

static int dbg_link_frame(const char *frame_name,
    struct frame *base_frame, struct vec *base_vec)
{
	struct frame *frame;
	struct obj *obj;

	assert(!base_vec || base_vec->frame == base_frame);
	frame = find_frame(frame_name);
	if (!frame) {
		yyerrorf("unknown frame \"%s\"", frame_name);
		return 0;
	}
	/* this can only fail in %frame */
	if (is_parent_of(frame, base_frame)) {
		yyerrorf("frame \"%s\" is a parent of \"%s\"",
		    frame->name, base_frame->name);
		return 0;
	}
	obj = new_obj(ot_frame);
	obj->base = base_vec;
	obj->frame = base_frame;
	obj->u.frame.ref = frame;
	connect_obj(base_frame, obj);
	if (!frame->active_ref)
		frame->active_ref = obj;
	return 1;
}


static int dbg_print(const struct expr *expr)
{
	const char *s;
	struct num num;

	s = eval_str(expr, curr_frame);
	if (s) {
		printf("%s\n", s);
		return 1;
	}
	num = eval_num(expr, curr_frame);
	if (is_undef(num))
		return 0;
	printf("%lg%s\n", num.n, str_unit(num));
	return 1;
}


static int dbg_meas(const char *name)
{
	const struct obj *obj;
	const struct inst *inst;
	struct coord a1, b1;
	char *s;

	obj = find_obj(frames, name);
	if (!obj) {
		yyerrorf("unknown object \"%s\"", name);
		return 0;
	}

	/* from fped.c:main */

	if (!pkg_name)
                pkg_name = stralloc("_");
        reporter = report_to_stderr;
	if (!instantiate())
		return 0;

	inst = find_meas_hint(obj);
	if (!inst) {
		yyerrorf("measurement \"%s\" was not instantiated", name);
		return 0;
	}

	project_meas(inst, &a1, &b1);
	s = format_len(obj->u.meas.label ? obj->u.meas.label : "",
	    dist_point(a1, b1), curr_unit);
	printf("%s\n", s);
	free(s);

	return 1;
}




/* Line 268 of yacc.c  */
#line 496 "y.tab.c"

/* Enabling traces.  */
#ifndef YYDEBUG
# define YYDEBUG 0
#endif

/* Enabling verbose error messages.  */
#ifdef YYERROR_VERBOSE
# undef YYERROR_VERBOSE
# define YYERROR_VERBOSE 1
#else
# define YYERROR_VERBOSE 0
#endif

/* Enabling the token table.  */
#ifndef YYTOKEN_TABLE
# define YYTOKEN_TABLE 0
#endif


/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     START_FPD = 258,
     START_EXPR = 259,
     START_VAR = 260,
     START_VALUES = 261,
     TOK_SET = 262,
     TOK_LOOP = 263,
     TOK_PACKAGE = 264,
     TOK_FRAME = 265,
     TOK_TABLE = 266,
     TOK_VEC = 267,
     TOK_PAD = 268,
     TOK_RPAD = 269,
     TOK_HOLE = 270,
     TOK_RECT = 271,
     TOK_LINE = 272,
     TOK_CIRC = 273,
     TOK_ARC = 274,
     TOK_MEAS = 275,
     TOK_MEASX = 276,
     TOK_MEASY = 277,
     TOK_UNIT = 278,
     TOK_NEXT = 279,
     TOK_NEXT_INVERTED = 280,
     TOK_MAX = 281,
     TOK_MAX_INVERTED = 282,
     TOK_DBG_DEL = 283,
     TOK_DBG_MOVE = 284,
     TOK_DBG_FRAME = 285,
     TOK_DBG_PRINT = 286,
     TOK_DBG_DUMP = 287,
     TOK_DBG_EXIT = 288,
     TOK_DBG_TSORT = 289,
     TOK_DBG_MEAS = 290,
     TOK_ALLOW_OVERLAP = 291,
     TOK_ALLOW_TOUCH = 292,
     NUMBER = 293,
     STRING = 294,
     ID = 295,
     LABEL = 296
   };
#endif
/* Tokens.  */
#define START_FPD 258
#define START_EXPR 259
#define START_VAR 260
#define START_VALUES 261
#define TOK_SET 262
#define TOK_LOOP 263
#define TOK_PACKAGE 264
#define TOK_FRAME 265
#define TOK_TABLE 266
#define TOK_VEC 267
#define TOK_PAD 268
#define TOK_RPAD 269
#define TOK_HOLE 270
#define TOK_RECT 271
#define TOK_LINE 272
#define TOK_CIRC 273
#define TOK_ARC 274
#define TOK_MEAS 275
#define TOK_MEASX 276
#define TOK_MEASY 277
#define TOK_UNIT 278
#define TOK_NEXT 279
#define TOK_NEXT_INVERTED 280
#define TOK_MAX 281
#define TOK_MAX_INVERTED 282
#define TOK_DBG_DEL 283
#define TOK_DBG_MOVE 284
#define TOK_DBG_FRAME 285
#define TOK_DBG_PRINT 286
#define TOK_DBG_DUMP 287
#define TOK_DBG_EXIT 288
#define TOK_DBG_TSORT 289
#define TOK_DBG_MEAS 290
#define TOK_ALLOW_OVERLAP 291
#define TOK_ALLOW_TOUCH 292
#define NUMBER 293
#define STRING 294
#define ID 295
#define LABEL 296




#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef union YYSTYPE
{

/* Line 293 of yacc.c  */
#line 426 "fpd.y"

	struct num num;
	char *str;
	const char *id;
	struct expr *expr;
	struct frame *frame;
	struct table *table;
	struct var *var;
	struct row *row;
	struct value *value;
	struct vec *vec;
	struct obj *obj;
	enum pad_type pt;
	enum meas_type mt;
	struct {
		int inverted;
		int max;
	} mo;
	struct {
		struct frame *frame;
		struct vec *vec;
	} qvec;
	struct qbase {
		struct vec *vec;
		struct frame_qual *qualifiers;
	} qbase;



/* Line 293 of yacc.c  */
#line 644 "y.tab.c"
} YYSTYPE;
# define YYSTYPE_IS_TRIVIAL 1
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
#endif


/* Copy the second part of user declarations.  */


/* Line 343 of yacc.c  */
#line 656 "y.tab.c"

#ifdef short
# undef short
#endif

#ifdef YYTYPE_UINT8
typedef YYTYPE_UINT8 yytype_uint8;
#else
typedef unsigned char yytype_uint8;
#endif

#ifdef YYTYPE_INT8
typedef YYTYPE_INT8 yytype_int8;
#elif (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
typedef signed char yytype_int8;
#else
typedef short int yytype_int8;
#endif

#ifdef YYTYPE_UINT16
typedef YYTYPE_UINT16 yytype_uint16;
#else
typedef unsigned short int yytype_uint16;
#endif

#ifdef YYTYPE_INT16
typedef YYTYPE_INT16 yytype_int16;
#else
typedef short int yytype_int16;
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif ! defined YYSIZE_T && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned int
# endif
#endif

#define YYSIZE_MAXIMUM ((YYSIZE_T) -1)

#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YYUSE(e) ((void) (e))
#else
# define YYUSE(e) /* empty */
#endif

/* Identity function, used to suppress warnings about constant conditions.  */
#ifndef lint
# define YYID(n) (n)
#else
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static int
YYID (int yyi)
#else
static int
YYID (yyi)
    int yyi;
#endif
{
  return yyi;
}
#endif

#if ! defined yyoverflow || YYERROR_VERBOSE

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#     ifndef EXIT_SUCCESS
#      define EXIT_SUCCESS 0
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's `empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (YYID (0))
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
       && ! ((defined YYMALLOC || defined malloc) \
	     && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef EXIT_SUCCESS
#    define EXIT_SUCCESS 0
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* ! defined yyoverflow || YYERROR_VERBOSE */


#if (! defined yyoverflow \
     && (! defined __cplusplus \
	 || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yytype_int16 yyss_alloc;
  YYSTYPE yyvs_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
      + YYSTACK_GAP_MAXIMUM)

# define YYCOPY_NEEDED 1

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
    do									\
      {									\
	YYSIZE_T yynewbytes;						\
	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
	Stack = &yyptr->Stack_alloc;					\
	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
	yyptr += yynewbytes / sizeof (*yyptr);				\
      }									\
    while (YYID (0))

#endif

#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
/* Copy COUNT objects from FROM to TO.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(To, From, Count) \
      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
#  else
#   define YYCOPY(To, From, Count)		\
      do					\
	{					\
	  YYSIZE_T yyi;				\
	  for (yyi = 0; yyi < (Count); yyi++)	\
	    (To)[yyi] = (From)[yyi];		\
	}					\
      while (YYID (0))
#  endif
# endif
#endif /* !YYCOPY_NEEDED */

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  21
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   230

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  54
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  52
/* YYNRULES -- Number of rules.  */
#define YYNRULES  123
/* YYNRULES -- Number of states.  */
#define YYNSTATES  217

/* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
#define YYUNDEFTOK  2
#define YYMAXUTOK   296

#define YYTRANSLATE(YYX)						\
  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)

/* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
static const yytype_uint8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      49,    50,    53,    47,    45,    48,    46,    52,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,    44,     2,     2,    51,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,    42,     2,    43,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
      35,    36,    37,    38,    39,    40,    41
};

#if YYDEBUG
/* YYPRHS[YYN] -- Index of the first RHS symbol of rule number YYN in
   YYRHS.  */
static const yytype_uint16 yyprhs[] =
{
       0,     0,     3,     4,     8,    11,    15,    18,    24,    29,
      33,    36,    39,    40,    42,    44,    46,    49,    52,    55,
      57,    59,    60,    63,    64,    71,    72,    74,    76,    79,
      81,    86,    93,    95,    98,   100,   103,   105,   108,   113,
     118,   122,   125,   128,   130,   132,   133,   139,   140,   144,
     148,   153,   154,   161,   163,   167,   169,   170,   171,   172,
     179,   181,   185,   187,   195,   197,   199,   201,   203,   206,
     209,   212,   214,   220,   226,   230,   235,   240,   245,   251,
     252,   257,   258,   260,   261,   263,   265,   268,   271,   273,
     276,   283,   285,   287,   291,   295,   297,   299,   301,   303,
     305,   307,   309,   310,   312,   313,   315,   316,   318,   320,
     322,   326,   330,   332,   336,   340,   342,   345,   347,   349,
     351,   355,   360,   361
};

/* YYRHS -- A `-1'-separated list of the rules' RHS.  */
static const yytype_int8 yyrhs[] =
{
      55,     0,    -1,    -1,     3,    56,    57,    -1,     4,   100,
      -1,     5,    40,   105,    -1,     6,    79,    -1,    63,    58,
      59,    66,    89,    -1,    63,    60,    66,    89,    -1,    63,
      67,    89,    -1,    63,    89,    -1,     9,    39,    -1,    -1,
      60,    -1,    61,    -1,    62,    -1,    61,    62,    -1,    62,
      61,    -1,    23,    40,    -1,    37,    -1,    36,    -1,    -1,
      63,    64,    -1,    -1,    10,    40,    42,    65,    66,    43,
      -1,    -1,    67,    -1,    68,    -1,    68,    67,    -1,    72,
      -1,     7,    40,    44,   100,    -1,     8,    40,    44,   100,
      45,   100,    -1,    81,    -1,    41,    81,    -1,    85,    -1,
      41,    85,    -1,    69,    -1,    28,    40,    -1,    28,    40,
      46,    40,    -1,    29,    40,    97,    40,    -1,    30,    40,
      83,    -1,    31,   100,    -1,    35,    40,    -1,    32,    -1,
      33,    -1,    -1,    34,    42,    70,    71,    43,    -1,    -1,
      71,    47,    40,    -1,    71,    48,    40,    -1,    71,    40,
      40,    97,    -1,    -1,    11,    73,    42,    74,    43,    76,
      -1,    75,    -1,    74,    45,    75,    -1,    40,    -1,    -1,
      -1,    -1,    42,    77,    79,    43,    78,    76,    -1,    80,
      -1,    79,    45,    80,    -1,   100,    -1,    12,    82,    49,
     100,    45,   100,    50,    -1,    51,    -1,    46,    -1,    40,
      -1,    82,    -1,    84,    51,    -1,    84,    40,    -1,    40,
      46,    -1,    86,    -1,    13,    39,    82,    82,    88,    -1,
      14,    39,    82,    82,    88,    -1,    15,    82,    82,    -1,
      16,    82,    82,    99,    -1,    17,    82,    82,    99,    -1,
      18,    82,    82,    99,    -1,    19,    82,    82,    82,    99,
      -1,    -1,    10,    40,    87,    82,    -1,    -1,    40,    -1,
      -1,    90,    -1,    92,    -1,    90,    91,    -1,    90,    69,
      -1,    92,    -1,    41,    92,    -1,    95,    98,    93,    96,
      93,    99,    -1,    94,    -1,    40,    -1,    40,    46,    40,
      -1,    40,    52,    93,    -1,    20,    -1,    21,    -1,    22,
      -1,    24,    -1,    25,    -1,    26,    -1,    27,    -1,    -1,
      38,    -1,    -1,    39,    -1,    -1,   100,    -1,   101,    -1,
     102,    -1,   101,    47,   102,    -1,   101,    48,   102,    -1,
     103,    -1,   102,    53,   103,    -1,   102,    52,   103,    -1,
     104,    -1,    48,   104,    -1,    38,    -1,    40,    -1,    39,
      -1,    49,   100,    50,    -1,    40,    49,   100,    50,    -1,
      -1,    44,    79,    -1
};

/* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
static const yytype_uint16 yyrline[] =
{
       0,   488,   488,   487,   496,   500,   505,   512,   513,   514,
     515,   519,   536,   537,   541,   542,   543,   544,   548,   564,
     568,   574,   575,   580,   579,   597,   598,   602,   603,   607,
     608,   616,   624,   625,   633,   634,   642,   646,   651,   656,
     661,   666,   671,   676,   683,   688,   687,   702,   703,   707,
     711,   724,   723,   741,   745,   756,   773,   777,   784,   776,
     800,   804,   815,   826,   842,   846,   854,   865,   870,   875,
     888,   899,   908,   917,   926,   932,   939,   946,   954,   963,
     962,   982,   985,  1002,  1003,  1007,  1012,  1017,  1021,  1025,
    1037,  1056,  1065,  1074,  1086,  1106,  1110,  1114,  1121,  1126,
    1131,  1136,  1144,  1147,  1154,  1157,  1164,  1167,  1174,  1181,
    1185,  1189,  1196,  1200,  1204,  1211,  1215,  1222,  1227,  1232,
    1237,  1241,  1259,  1262
};
#endif

#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "$end", "error", "$undefined", "START_FPD", "START_EXPR", "START_VAR",
  "START_VALUES", "TOK_SET", "TOK_LOOP", "TOK_PACKAGE", "TOK_FRAME",
  "TOK_TABLE", "TOK_VEC", "TOK_PAD", "TOK_RPAD", "TOK_HOLE", "TOK_RECT",
  "TOK_LINE", "TOK_CIRC", "TOK_ARC", "TOK_MEAS", "TOK_MEASX", "TOK_MEASY",
  "TOK_UNIT", "TOK_NEXT", "TOK_NEXT_INVERTED", "TOK_MAX",
  "TOK_MAX_INVERTED", "TOK_DBG_DEL", "TOK_DBG_MOVE", "TOK_DBG_FRAME",
  "TOK_DBG_PRINT", "TOK_DBG_DUMP", "TOK_DBG_EXIT", "TOK_DBG_TSORT",
  "TOK_DBG_MEAS", "TOK_ALLOW_OVERLAP", "TOK_ALLOW_TOUCH", "NUMBER",
  "STRING", "ID", "LABEL", "'{'", "'}'", "'='", "','", "'.'", "'+'", "'-'",
  "'('", "')'", "'@'", "'/'", "'*'", "$accept", "all", "$@1", "fpd",
  "part_name", "opt_setup", "setup", "unit", "allow", "frame_defs",
  "frame_def", "$@2", "opt_frame_items", "frame_items", "frame_item",
  "debug_item", "$@3", "sort_items", "table", "@4", "vars", "var", "rows",
  "@5", "$@6", "row", "value", "vec", "base", "qualified_base",
  "frame_qualifier", "object", "obj", "@7", "pad_type", "opt_measurements",
  "measurements", "meas", "unlabeled_meas", "qbase", "qbase_unchecked",
  "meas_type", "meas_op", "opt_num", "opt_string", "opt_expr", "expr",
  "add_expr", "mult_expr", "unary_expr", "primary_expr", "opt_value_list", 0
};
#endif

# ifdef YYPRINT
/* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
   token YYLEX-NUM.  */
static const yytype_uint16 yytoknum[] =
{
       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
     275,   276,   277,   278,   279,   280,   281,   282,   283,   284,
     285,   286,   287,   288,   289,   290,   291,   292,   293,   294,
     295,   296,   123,   125,    61,    44,    46,    43,    45,    40,
      41,    64,    47,    42
};
# endif

/* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
static const yytype_uint8 yyr1[] =
{
       0,    54,    56,    55,    55,    55,    55,    57,    57,    57,
      57,    58,    59,    59,    60,    60,    60,    60,    61,    62,
      62,    63,    63,    65,    64,    66,    66,    67,    67,    68,
      68,    68,    68,    68,    68,    68,    68,    69,    69,    69,
      69,    69,    69,    69,    69,    70,    69,    71,    71,    71,
      71,    73,    72,    74,    74,    75,    76,    77,    78,    76,
      79,    79,    80,    81,    82,    82,    82,    83,    83,    83,
      84,    85,    86,    86,    86,    86,    86,    86,    86,    87,
      86,    88,    88,    89,    89,    90,    90,    90,    91,    91,
      92,    93,    94,    94,    94,    95,    95,    95,    96,    96,
      96,    96,    97,    97,    98,    98,    99,    99,   100,   101,
     101,   101,   102,   102,   102,   103,   103,   104,   104,   104,
     104,   104,   105,   105
};

/* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
static const yytype_uint8 yyr2[] =
{
       0,     2,     0,     3,     2,     3,     2,     5,     4,     3,
       2,     2,     0,     1,     1,     1,     2,     2,     2,     1,
       1,     0,     2,     0,     6,     0,     1,     1,     2,     1,
       4,     6,     1,     2,     1,     2,     1,     2,     4,     4,
       3,     2,     2,     1,     1,     0,     5,     0,     3,     3,
       4,     0,     6,     1,     3,     1,     0,     0,     0,     6,
       1,     3,     1,     7,     1,     1,     1,     1,     2,     2,
       2,     1,     5,     5,     3,     4,     4,     4,     5,     0,
       4,     0,     1,     0,     1,     1,     2,     2,     1,     2,
       6,     1,     1,     3,     3,     1,     1,     1,     1,     1,
       1,     1,     0,     1,     0,     1,     0,     1,     1,     1,
       3,     3,     1,     3,     3,     1,     2,     1,     1,     1,
       3,     4,     0,     2
};

/* YYDEFACT[STATE-NAME] -- Default reduction number in state STATE-NUM.
   Performed when YYTABLE doesn't specify something else to do.  Zero
   means the default is an error.  */
static const yytype_uint8 yydefact[] =
{
       0,     2,     0,     0,     0,     0,    21,   117,   119,   118,
       0,     0,     4,   108,   109,   112,   115,   122,     6,    60,
      62,     1,     3,    83,     0,   116,     0,     0,     0,     0,
       0,     0,     5,     0,     0,     0,     0,     0,    51,     0,
       0,     0,     0,     0,     0,     0,     0,    95,    96,    97,
       0,     0,     0,     0,     0,    43,    44,     0,     0,    20,
      19,     0,    12,    25,    14,    15,    22,    83,    27,    36,
      29,    32,    34,    71,    10,    84,    85,   104,     0,   120,
     110,   111,   114,   113,   123,    61,     0,     0,    11,    79,
       0,    66,    65,    64,     0,     0,     0,     0,     0,     0,
       0,     0,    18,    37,   102,     0,    41,    45,    42,     0,
      33,    35,    25,    13,    83,    26,    16,    17,     9,    28,
       0,    87,    86,    88,   105,     0,   121,     0,     0,    23,
       0,     0,     0,     0,     0,    74,   106,   106,   106,     0,
       0,   103,     0,    66,    67,    40,     0,    47,    79,    83,
       8,    89,    92,     0,    91,    30,     0,    25,    80,    55,
       0,    53,     0,    81,    81,    75,   107,    76,    77,   106,
      38,    39,    70,    69,    68,     0,     7,     0,     0,    98,
      99,   100,   101,     0,     0,     0,    56,     0,     0,    82,
      72,    73,    78,     0,    46,     0,     0,    93,    94,   106,
      31,    24,    57,    52,    54,     0,   102,    48,    49,    90,
       0,    63,    50,     0,    58,    56,    59
};

/* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int16 yydefgoto[] =
{
      -1,     5,     6,    22,    62,   112,    63,    64,    65,    23,
      66,   157,   114,   115,    68,    69,   147,   175,    70,    90,
     160,   161,   203,   210,   215,    18,    19,    71,    94,   145,
     146,    72,    73,   130,   190,    74,    75,   122,    76,   153,
     154,    77,   183,   142,   125,   165,   166,    13,    14,    15,
      16,    32
};

/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
   STATE-NUM.  */
#define YYPACT_NINF -131
static const yytype_int16 yypact[] =
{
     106,  -131,    29,   -27,    29,    30,  -131,  -131,  -131,    -8,
      35,    29,  -131,    23,   -36,  -131,  -131,     6,    -1,  -131,
    -131,  -131,  -131,   124,    29,  -131,    -3,    29,    29,    29,
      29,    29,  -131,    29,    36,    40,    46,    73,  -131,     0,
      80,    84,     0,     0,     0,     0,     0,  -131,  -131,  -131,
      87,    88,   108,   109,    29,  -131,  -131,    82,   110,  -131,
    -131,    89,    -9,   159,    52,   128,  -131,    76,   159,  -131,
    -131,  -131,  -131,  -131,  -131,     4,  -131,   123,    79,  -131,
     -36,   -36,  -131,  -131,    -1,  -131,   119,   120,  -131,   126,
     137,  -131,  -131,  -131,   132,     0,     0,     0,     0,     0,
       0,     0,  -131,   138,   145,     3,  -131,  -131,  -131,   155,
    -131,  -131,   159,  -131,    76,  -131,  -131,  -131,  -131,  -131,
      76,  -131,  -131,  -131,  -131,   156,  -131,    29,    29,  -131,
       0,   157,    29,     0,     0,  -131,    29,    29,    29,     0,
     158,  -131,   161,   139,  -131,  -131,   -28,  -131,  -131,    76,
    -131,  -131,   -31,    90,  -131,  -131,   154,   159,  -131,  -131,
      38,  -131,   160,   162,   162,  -131,  -131,  -131,  -131,    29,
    -131,  -131,  -131,  -131,  -131,    39,  -131,   163,   156,  -131,
    -131,  -131,  -131,   156,    29,   164,   167,   157,    29,  -131,
    -131,  -131,  -131,   166,  -131,   170,   171,  -131,  -131,    29,
    -131,  -131,  -131,  -131,  -131,   165,   145,  -131,  -131,  -131,
      29,  -131,  -131,    75,  -131,   167,  -131
};

/* YYPGOTO[NTERM-NUM].  */
static const yytype_int16 yypgoto[] =
{
    -131,  -131,  -131,  -131,  -131,  -131,   142,   147,   149,  -131,
    -131,  -131,   -94,    -4,  -131,   141,  -131,  -131,  -131,  -131,
    -131,    27,     2,  -131,  -131,   -30,   185,   168,   -39,  -131,
    -131,   169,  -131,  -131,    55,   -59,  -131,  -131,   -55,  -130,
    -131,  -131,  -131,    14,  -131,  -127,    -2,  -131,    65,    92,
     211,  -131
};

/* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
   positive, shift that token.  If negative, reduce the rule which
   number is the opposite.  If YYTABLE_NINF, syntax error.  */
#define YYTABLE_NINF -1
static const yytype_uint8 yytable[] =
{
      12,    84,    20,    97,    98,    99,   100,   101,   118,    26,
     167,   168,   173,    17,    50,   177,    29,    30,   149,    67,
     123,   178,    78,   174,    47,    48,    49,    59,    60,    20,
      21,    20,    51,    52,    53,    54,    55,    56,    57,    58,
      91,    24,   192,   143,    33,   120,    92,    79,   198,    92,
      31,    93,   106,   199,    93,   150,   133,   134,   135,   136,
     137,   138,   139,   185,   119,   151,   144,     7,     8,     9,
      27,    28,   209,     7,     8,     9,    86,    10,    11,   193,
      87,   186,   194,   187,    11,    88,   195,   196,    59,    60,
     176,   158,    80,    81,   163,   164,    47,    48,    49,   109,
     169,    39,    40,    41,    42,    43,    44,    45,    46,     1,
       2,     3,     4,    89,   179,   180,   181,   182,   214,    95,
      33,    82,    83,    96,   107,   155,   156,   102,   103,   126,
     162,    34,    35,    36,    37,    38,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,   104,   105,
     108,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      59,    60,   124,   127,   128,    61,    34,    35,   129,   109,
      38,    39,    40,    41,    42,    43,    44,    45,    46,   131,
     213,   132,   200,   141,   140,   172,   205,    51,    52,    53,
      54,    55,    56,    57,    58,   148,   152,   159,   170,   184,
      61,   171,   189,   197,   113,   188,   206,   201,    20,   202,
     207,   208,   117,   116,   204,   211,   121,   216,    85,   191,
     212,    25,     0,     0,     0,     0,     0,     0,     0,   110,
     111
};

#define yypact_value_is_default(yystate) \
  ((yystate) == (-131))

#define yytable_value_is_error(yytable_value) \
  YYID (0)

static const yytype_int16 yycheck[] =
{
       2,    31,     4,    42,    43,    44,    45,    46,    67,    11,
     137,   138,    40,    40,    23,    46,    52,    53,   112,    23,
      75,    52,    24,    51,    20,    21,    22,    36,    37,    31,
       0,    33,    28,    29,    30,    31,    32,    33,    34,    35,
      40,    49,   169,    40,    45,    41,    46,    50,   178,    46,
      44,    51,    54,   183,    51,   114,    95,    96,    97,    98,
      99,   100,   101,   157,    68,   120,   105,    38,    39,    40,
      47,    48,   199,    38,    39,    40,    40,    48,    49,    40,
      40,    43,    43,    45,    49,    39,    47,    48,    36,    37,
     149,   130,    27,    28,   133,   134,    20,    21,    22,    10,
     139,    12,    13,    14,    15,    16,    17,    18,    19,     3,
       4,     5,     6,    40,    24,    25,    26,    27,    43,    39,
      45,    29,    30,    39,    42,   127,   128,    40,    40,    50,
     132,     7,     8,     9,    10,    11,    12,    13,    14,    15,
      16,    17,    18,    19,    20,    21,    22,    23,    40,    40,
      40,    23,    28,    29,    30,    31,    32,    33,    34,    35,
      36,    37,    39,    44,    44,    41,     7,     8,    42,    10,
      11,    12,    13,    14,    15,    16,    17,    18,    19,    42,
     210,    49,   184,    38,    46,    46,   188,    28,    29,    30,
      31,    32,    33,    34,    35,    40,    40,    40,    40,    45,
      41,    40,    40,    40,    62,    45,    40,    43,   210,    42,
      40,    40,    65,    64,   187,    50,    75,   215,    33,   164,
     206,    10,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    61,
      61
};

/* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
   symbol of state STATE-NUM.  */
static const yytype_uint8 yystos[] =
{
       0,     3,     4,     5,     6,    55,    56,    38,    39,    40,
      48,    49,   100,   101,   102,   103,   104,    40,    79,    80,
     100,     0,    57,    63,    49,   104,   100,    47,    48,    52,
      53,    44,   105,    45,     7,     8,     9,    10,    11,    12,
      13,    14,    15,    16,    17,    18,    19,    20,    21,    22,
      23,    28,    29,    30,    31,    32,    33,    34,    35,    36,
      37,    41,    58,    60,    61,    62,    64,    67,    68,    69,
      72,    81,    85,    86,    89,    90,    92,    95,   100,    50,
     102,   102,   103,   103,    79,    80,    40,    40,    39,    40,
      73,    40,    46,    51,    82,    39,    39,    82,    82,    82,
      82,    82,    40,    40,    40,    40,   100,    42,    40,    10,
      81,    85,    59,    60,    66,    67,    62,    61,    89,    67,
      41,    69,    91,    92,    39,    98,    50,    44,    44,    42,
      87,    42,    49,    82,    82,    82,    82,    82,    82,    82,
      46,    38,    97,    40,    82,    83,    84,    70,    40,    66,
      89,    92,    40,    93,    94,   100,   100,    65,    82,    40,
      74,    75,   100,    82,    82,    99,   100,    99,    99,    82,
      40,    40,    46,    40,    51,    71,    89,    46,    52,    24,
      25,    26,    27,    96,    45,    66,    43,    45,    45,    40,
      88,    88,    99,    40,    43,    47,    48,    40,    93,    93,
     100,    43,    42,    76,    75,   100,    40,    40,    40,    99,
      77,    50,    97,    79,    43,    78,    76
};

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		(-2)
#define YYEOF		0

#define YYACCEPT	goto yyacceptlab
#define YYABORT		goto yyabortlab
#define YYERROR		goto yyerrorlab


/* Like YYERROR except do call yyerror.  This remains here temporarily
   to ease the transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  However,
   YYFAIL appears to be in use.  Nevertheless, it is formally deprecated
   in Bison 2.4.2's NEWS entry, where a plan to phase it out is
   discussed.  */

#define YYFAIL		goto yyerrlab
#if defined YYFAIL
  /* This is here to suppress warnings from the GCC cpp's
     -Wunused-macros.  Normally we don't worry about that warning, but
     some users do, and we want to make it easy for users to remove
     YYFAIL uses, which will produce warnings from Bison 2.5.  */
#endif

#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)					\
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    {								\
      yychar = (Token);						\
      yylval = (Value);						\
      YYPOPSTACK (1);						\
      goto yybackup;						\
    }								\
  else								\
    {								\
      yyerror (YY_("syntax error: cannot back up")); \
      YYERROR;							\
    }								\
while (YYID (0))


#define YYTERROR	1
#define YYERRCODE	256


/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
   If N is 0, then set CURRENT to the empty location which ends
   the previous symbol: RHS[0] (always defined).  */

#define YYRHSLOC(Rhs, K) ((Rhs)[K])
#ifndef YYLLOC_DEFAULT
# define YYLLOC_DEFAULT(Current, Rhs, N)				\
    do									\
      if (YYID (N))                                                    \
	{								\
	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
	}								\
      else								\
	{								\
	  (Current).first_line   = (Current).last_line   =		\
	    YYRHSLOC (Rhs, 0).last_line;				\
	  (Current).first_column = (Current).last_column =		\
	    YYRHSLOC (Rhs, 0).last_column;				\
	}								\
    while (YYID (0))
#endif


/* This macro is provided for backward compatibility. */

#ifndef YY_LOCATION_PRINT
# define YY_LOCATION_PRINT(File, Loc) ((void) 0)
#endif


/* YYLEX -- calling `yylex' with the right arguments.  */

#ifdef YYLEX_PARAM
# define YYLEX yylex (YYLEX_PARAM)
#else
# define YYLEX yylex ()
#endif

/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)			\
do {						\
  if (yydebug)					\
    YYFPRINTF Args;				\
} while (YYID (0))

# define YY_SYMBOL_PRINT(Title, Type, Value, Location)			  \
do {									  \
  if (yydebug)								  \
    {									  \
      YYFPRINTF (stderr, "%s ", Title);					  \
      yy_symbol_print (stderr,						  \
		  Type, Value); \
      YYFPRINTF (stderr, "\n");						  \
    }									  \
} while (YYID (0))


/*--------------------------------.
| Print this symbol on YYOUTPUT.  |
`--------------------------------*/

/*ARGSUSED*/
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
#else
static void
yy_symbol_value_print (yyoutput, yytype, yyvaluep)
    FILE *yyoutput;
    int yytype;
    YYSTYPE const * const yyvaluep;
#endif
{
  if (!yyvaluep)
    return;
# ifdef YYPRINT
  if (yytype < YYNTOKENS)
    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
# else
  YYUSE (yyoutput);
# endif
  switch (yytype)
    {
      default:
	break;
    }
}


/*--------------------------------.
| Print this symbol on YYOUTPUT.  |
`--------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
#else
static void
yy_symbol_print (yyoutput, yytype, yyvaluep)
    FILE *yyoutput;
    int yytype;
    YYSTYPE const * const yyvaluep;
#endif
{
  if (yytype < YYNTOKENS)
    YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
  else
    YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);

  yy_symbol_value_print (yyoutput, yytype, yyvaluep);
  YYFPRINTF (yyoutput, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
#else
static void
yy_stack_print (yybottom, yytop)
    yytype_int16 *yybottom;
    yytype_int16 *yytop;
#endif
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)				\
do {								\
  if (yydebug)							\
    yy_stack_print ((Bottom), (Top));				\
} while (YYID (0))


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_reduce_print (YYSTYPE *yyvsp, int yyrule)
#else
static void
yy_reduce_print (yyvsp, yyrule)
    YYSTYPE *yyvsp;
    int yyrule;
#endif
{
  int yynrhs = yyr2[yyrule];
  int yyi;
  unsigned long int yylno = yyrline[yyrule];
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu):\n",
	     yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
		       &(yyvsp[(yyi + 1) - (yynrhs)])
		       		       );
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)		\
do {					\
  if (yydebug)				\
    yy_reduce_print (yyvsp, Rule); \
} while (YYID (0))

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args)
# define YY_SYMBOL_PRINT(Title, Type, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef	YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif


#if YYERROR_VERBOSE

# ifndef yystrlen
#  if defined __GLIBC__ && defined _STRING_H
#   define yystrlen strlen
#  else
/* Return the length of YYSTR.  */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static YYSIZE_T
yystrlen (const char *yystr)
#else
static YYSIZE_T
yystrlen (yystr)
    const char *yystr;
#endif
{
  YYSIZE_T yylen;
  for (yylen = 0; yystr[yylen]; yylen++)
    continue;
  return yylen;
}
#  endif
# endif

# ifndef yystpcpy
#  if defined __GLIBC__ && defined _STRING_H && defined _GNU_SOURCE
#   define yystpcpy stpcpy
#  else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static char *
yystpcpy (char *yydest, const char *yysrc)
#else
static char *
yystpcpy (yydest, yysrc)
    char *yydest;
    const char *yysrc;
#endif
{
  char *yyd = yydest;
  const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
#  endif
# endif

# ifndef yytnamerr
/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
   quotes and backslashes, so that it's suitable for yyerror.  The
   heuristic is that double-quoting is unnecessary unless the string
   contains an apostrophe, a comma, or backslash (other than
   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
   null, do not copy; instead, return the length of what the result
   would have been.  */
static YYSIZE_T
yytnamerr (char *yyres, const char *yystr)
{
  if (*yystr == '"')
    {
      YYSIZE_T yyn = 0;
      char const *yyp = yystr;

      for (;;)
	switch (*++yyp)
	  {
	  case '\'':
	  case ',':
	    goto do_not_strip_quotes;

	  case '\\':
	    if (*++yyp != '\\')
	      goto do_not_strip_quotes;
	    /* Fall through.  */
	  default:
	    if (yyres)
	      yyres[yyn] = *yyp;
	    yyn++;
	    break;

	  case '"':
	    if (yyres)
	      yyres[yyn] = '\0';
	    return yyn;
	  }
    do_not_strip_quotes: ;
    }

  if (! yyres)
    return yystrlen (yystr);

  return yystpcpy (yyres, yystr) - yyres;
}
# endif

/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
   about the unexpected token YYTOKEN for the state stack whose top is
   YYSSP.

   Return 0 if *YYMSG was successfully written.  Return 1 if *YYMSG is
   not large enough to hold the message.  In that case, also set
   *YYMSG_ALLOC to the required number of bytes.  Return 2 if the
   required number of bytes is too large to store.  */
static int
yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                yytype_int16 *yyssp, int yytoken)
{
  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytoken]);
  YYSIZE_T yysize = yysize0;
  YYSIZE_T yysize1;
  enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
  /* Internationalized format string. */
  const char *yyformat = 0;
  /* Arguments of yyformat. */
  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
  /* Number of reported tokens (one for the "unexpected", one per
     "expected"). */
  int yycount = 0;

  /* There are many possibilities here to consider:
     - Assume YYFAIL is not used.  It's too flawed to consider.  See
       <http://lists.gnu.org/archive/html/bison-patches/2009-12/msg00024.html>
       for details.  YYERROR is fine as it does not invoke this
       function.
     - If this state is a consistent state with a default action, then
       the only way this function was invoked is if the default action
       is an error action.  In that case, don't check for expected
       tokens because there are none.
     - The only way there can be no lookahead present (in yychar) is if
       this state is a consistent state with a default action.  Thus,
       detecting the absence of a lookahead is sufficient to determine
       that there is no unexpected or expected token to report.  In that
       case, just report a simple "syntax error".
     - Don't assume there isn't a lookahead just because this state is a
       consistent state with a default action.  There might have been a
       previous inconsistent state, consistent state with a non-default
       action, or user semantic action that manipulated yychar.
     - Of course, the expected token list depends on states to have
       correct lookahead information, and it depends on the parser not
       to perform extra reductions after fetching a lookahead from the
       scanner and before detecting a syntax error.  Thus, state merging
       (from LALR or IELR) and default reductions corrupt the expected
       token list.  However, the list is correct for canonical LR with
       one exception: it will still contain any token that will not be
       accepted due to an error action in a later state.
  */
  if (yytoken != YYEMPTY)
    {
      int yyn = yypact[*yyssp];
      yyarg[yycount++] = yytname[yytoken];
      if (!yypact_value_is_default (yyn))
        {
          /* Start YYX at -YYN if negative to avoid negative indexes in
             YYCHECK.  In other words, skip the first -YYN actions for
             this state because they are default actions.  */
          int yyxbegin = yyn < 0 ? -yyn : 0;
          /* Stay within bounds of both yycheck and yytname.  */
          int yychecklim = YYLAST - yyn + 1;
          int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
          int yyx;

          for (yyx = yyxbegin; yyx < yyxend; ++yyx)
            if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR
                && !yytable_value_is_error (yytable[yyx + yyn]))
              {
                if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
                  {
                    yycount = 1;
                    yysize = yysize0;
                    break;
                  }
                yyarg[yycount++] = yytname[yyx];
                yysize1 = yysize + yytnamerr (0, yytname[yyx]);
                if (! (yysize <= yysize1
                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
                  return 2;
                yysize = yysize1;
              }
        }
    }

  switch (yycount)
    {
# define YYCASE_(N, S)                      \
      case N:                               \
        yyformat = S;                       \
      break
      YYCASE_(0, YY_("syntax error"));
      YYCASE_(1, YY_("syntax error, unexpected %s"));
      YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
      YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
      YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
      YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
# undef YYCASE_
    }

  yysize1 = yysize + yystrlen (yyformat);
  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
    return 2;
  yysize = yysize1;

  if (*yymsg_alloc < yysize)
    {
      *yymsg_alloc = 2 * yysize;
      if (! (yysize <= *yymsg_alloc
             && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
        *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
      return 1;
    }

  /* Avoid sprintf, as that infringes on the user's name space.
     Don't have undefined behavior even if the translation
     produced a string with the wrong number of "%s"s.  */
  {
    char *yyp = *yymsg;
    int yyi = 0;
    while ((*yyp = *yyformat) != '\0')
      if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
        {
          yyp += yytnamerr (yyp, yyarg[yyi++]);
          yyformat += 2;
        }
      else
        {
          yyp++;
          yyformat++;
        }
  }
  return 0;
}
#endif /* YYERROR_VERBOSE */

/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

/*ARGSUSED*/
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
#else
static void
yydestruct (yymsg, yytype, yyvaluep)
    const char *yymsg;
    int yytype;
    YYSTYPE *yyvaluep;
#endif
{
  YYUSE (yyvaluep);

  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);

  switch (yytype)
    {

      default:
	break;
    }
}


/* Prevent warnings from -Wmissing-prototypes.  */
#ifdef YYPARSE_PARAM
#if defined __STDC__ || defined __cplusplus
int yyparse (void *YYPARSE_PARAM);
#else
int yyparse ();
#endif
#else /* ! YYPARSE_PARAM */
#if defined __STDC__ || defined __cplusplus
int yyparse (void);
#else
int yyparse ();
#endif
#endif /* ! YYPARSE_PARAM */


/* The lookahead symbol.  */
int yychar;

/* The semantic value of the lookahead symbol.  */
YYSTYPE yylval;

/* Number of syntax errors so far.  */
int yynerrs;


/*----------.
| yyparse.  |
`----------*/

#ifdef YYPARSE_PARAM
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
int
yyparse (void *YYPARSE_PARAM)
#else
int
yyparse (YYPARSE_PARAM)
    void *YYPARSE_PARAM;
#endif
#else /* ! YYPARSE_PARAM */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
int
yyparse (void)
#else
int
yyparse ()

#endif
#endif
{
    int yystate;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus;

    /* The stacks and their tools:
       `yyss': related to states.
       `yyvs': related to semantic values.

       Refer to the stacks thru separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* The state stack.  */
    yytype_int16 yyssa[YYINITDEPTH];
    yytype_int16 *yyss;
    yytype_int16 *yyssp;

    /* The semantic value stack.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs;
    YYSTYPE *yyvsp;

    YYSIZE_T yystacksize;

  int yyn;
  int yyresult;
  /* Lookahead token as an internal (translated) token number.  */
  int yytoken;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;

#if YYERROR_VERBOSE
  /* Buffer for error messages, and its allocated size.  */
  char yymsgbuf[128];
  char *yymsg = yymsgbuf;
  YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
#endif

#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  yytoken = 0;
  yyss = yyssa;
  yyvs = yyvsa;
  yystacksize = YYINITDEPTH;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY; /* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */
  yyssp = yyss;
  yyvsp = yyvs;

  goto yysetstate;

/*------------------------------------------------------------.
| yynewstate -- Push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
 yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;

 yysetstate:
  *yyssp = yystate;

  if (yyss + yystacksize - 1 <= yyssp)
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYSIZE_T yysize = yyssp - yyss + 1;

#ifdef yyoverflow
      {
	/* Give user a chance to reallocate the stack.  Use copies of
	   these so that the &'s don't force the real ones into
	   memory.  */
	YYSTYPE *yyvs1 = yyvs;
	yytype_int16 *yyss1 = yyss;

	/* Each stack pointer address is followed by the size of the
	   data in use in that stack, in bytes.  This used to be a
	   conditional around just the two extra args, but that might
	   be undefined if yyoverflow is a macro.  */
	yyoverflow (YY_("memory exhausted"),
		    &yyss1, yysize * sizeof (*yyssp),
		    &yyvs1, yysize * sizeof (*yyvsp),
		    &yystacksize);

	yyss = yyss1;
	yyvs = yyvs1;
      }
#else /* no yyoverflow */
# ifndef YYSTACK_RELOCATE
      goto yyexhaustedlab;
# else
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
	goto yyexhaustedlab;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
	yystacksize = YYMAXDEPTH;

      {
	yytype_int16 *yyss1 = yyss;
	union yyalloc *yyptr =
	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
	if (! yyptr)
	  goto yyexhaustedlab;
	YYSTACK_RELOCATE (yyss_alloc, yyss);
	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
#  undef YYSTACK_RELOCATE
	if (yyss1 != yyssa)
	  YYSTACK_FREE (yyss1);
      }
# endif
#endif /* no yyoverflow */

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;

      YYDPRINTF ((stderr, "Stack size increased to %lu\n",
		  (unsigned long int) yystacksize));

      if (yyss + yystacksize - 1 <= yyssp)
	YYABORT;
    }

  YYDPRINTF ((stderr, "Entering state %d\n", yystate));

  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;

/*-----------.
| yybackup.  |
`-----------*/
yybackup:

  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yypact_value_is_default (yyn))
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token: "));
      yychar = YYLEX;
    }

  if (yychar <= YYEOF)
    {
      yychar = yytoken = YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yytable_value_is_error (yyn))
        goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);

  /* Discard the shifted token.  */
  yychar = YYEMPTY;

  yystate = yyn;
  *++yyvsp = yylval;

  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- Do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     `$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];


  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
        case 2:

/* Line 1806 of yacc.c  */
#line 488 "fpd.y"
    {
			frames = zalloc_type(struct frame);
			set_frame(frames);
			id_sin = unique("sin");
			id_cos = unique("cos");
			id_sqrt = unique("sqrt");
		}
    break;

  case 4:

/* Line 1806 of yacc.c  */
#line 497 "fpd.y"
    {
			expr_result = (yyvsp[(2) - (2)].expr);
		}
    break;

  case 5:

/* Line 1806 of yacc.c  */
#line 501 "fpd.y"
    {
			var_id = (yyvsp[(2) - (3)].id);
			var_value_list = (yyvsp[(3) - (3)].value);
		}
    break;

  case 6:

/* Line 1806 of yacc.c  */
#line 506 "fpd.y"
    {
			var_value_list = (yyvsp[(2) - (2)].value);
		}
    break;

  case 11:

/* Line 1806 of yacc.c  */
#line 520 "fpd.y"
    {
			const char *p;

			if (!*(yyvsp[(2) - (2)].str)) {
				yyerrorf("invalid package name");
				YYABORT;
			}
			for (p = (yyvsp[(2) - (2)].str); *p; *p++)
				if (*p < 32 || *p > 126) {
					yyerrorf("invalid package name");
					YYABORT;
				}
			pkg_name = (yyvsp[(2) - (2)].str);
		}
    break;

  case 18:

/* Line 1806 of yacc.c  */
#line 549 "fpd.y"
    {
			if (!strcmp((yyvsp[(2) - (2)].id), "mm"))
				curr_unit = curr_unit_mm;
			else if (!strcmp((yyvsp[(2) - (2)].id), "mil"))
				curr_unit = curr_unit_mil;
			else if (!strcmp((yyvsp[(2) - (2)].id), "auto"))
				curr_unit = curr_unit_auto;
			else {
				yyerrorf("unrecognized unit \"%s\"", (yyvsp[(2) - (2)].id));
				YYABORT;
			}
		}
    break;

  case 19:

/* Line 1806 of yacc.c  */
#line 565 "fpd.y"
    {
			allow_overlap = ao_touch;
		}
    break;

  case 20:

/* Line 1806 of yacc.c  */
#line 569 "fpd.y"
    {
			allow_overlap = ao_any;
		}
    break;

  case 23:

/* Line 1806 of yacc.c  */
#line 580 "fpd.y"
    {
			if (find_frame((yyvsp[(2) - (3)].id))) {
				yyerrorf("duplicate frame \"%s\"", (yyvsp[(2) - (3)].id));
				YYABORT;
			}
			curr_frame = zalloc_type(struct frame);
			curr_frame->name = (yyvsp[(2) - (3)].id);
			set_frame(curr_frame);
			curr_frame->next = frames->next;
			frames->next = curr_frame;
		}
    break;

  case 24:

/* Line 1806 of yacc.c  */
#line 592 "fpd.y"
    {
			set_frame(frames);
		}
    break;

  case 30:

/* Line 1806 of yacc.c  */
#line 609 "fpd.y"
    {
			if (find_var(curr_frame, (yyvsp[(2) - (4)].id))) {
				yyerrorf("duplicate variable \"%s\"", (yyvsp[(2) - (4)].id));
				YYABORT;
			}
			make_var((yyvsp[(2) - (4)].id), (yyvsp[(4) - (4)].expr));
		}
    break;

  case 31:

/* Line 1806 of yacc.c  */
#line 617 "fpd.y"
    {
			if (find_var(curr_frame, (yyvsp[(2) - (6)].id))) {
				yyerrorf("duplicate variable \"%s\"", (yyvsp[(2) - (6)].id));
				YYABORT;
			}
			make_loop((yyvsp[(2) - (6)].id), (yyvsp[(4) - (6)].expr), (yyvsp[(6) - (6)].expr));
		}
    break;

  case 33:

/* Line 1806 of yacc.c  */
#line 626 "fpd.y"
    {
			if (find_label(curr_frame, (yyvsp[(1) - (2)].id))) {
				yyerrorf("duplicate label \"%s\"", (yyvsp[(1) - (2)].id));
				YYABORT;
			}
			(yyvsp[(2) - (2)].vec)->name = (yyvsp[(1) - (2)].id);
		}
    break;

  case 35:

/* Line 1806 of yacc.c  */
#line 635 "fpd.y"
    {
			if (find_label(curr_frame, (yyvsp[(1) - (2)].id))) {
				yyerrorf("duplicate label \"%s\"", (yyvsp[(1) - (2)].id));
				YYABORT;
			}
			(yyvsp[(2) - (2)].obj)->name = (yyvsp[(1) - (2)].id);
		}
    break;

  case 37:

/* Line 1806 of yacc.c  */
#line 647 "fpd.y"
    {
			if (!dbg_delete(NULL, (yyvsp[(2) - (2)].id)))
				YYABORT;
		}
    break;

  case 38:

/* Line 1806 of yacc.c  */
#line 652 "fpd.y"
    {
			if (!dbg_delete((yyvsp[(2) - (4)].id), (yyvsp[(4) - (4)].id)))
				YYABORT;
		}
    break;

  case 39:

/* Line 1806 of yacc.c  */
#line 657 "fpd.y"
    {
			if (!dbg_move((yyvsp[(2) - (4)].id), (yyvsp[(3) - (4)].num).n, (yyvsp[(4) - (4)].id)))
				YYABORT;
		}
    break;

  case 40:

/* Line 1806 of yacc.c  */
#line 662 "fpd.y"
    {
			if (!dbg_link_frame((yyvsp[(2) - (3)].id), (yyvsp[(3) - (3)].qvec).frame, (yyvsp[(3) - (3)].qvec).vec))
				YYABORT;
		}
    break;

  case 41:

/* Line 1806 of yacc.c  */
#line 667 "fpd.y"
    {
			if (!dbg_print((yyvsp[(2) - (2)].expr)))
				YYABORT;
		}
    break;

  case 42:

/* Line 1806 of yacc.c  */
#line 672 "fpd.y"
    {
			if (!dbg_meas((yyvsp[(2) - (2)].id)))
				YYABORT;
		}
    break;

  case 43:

/* Line 1806 of yacc.c  */
#line 677 "fpd.y"
    {
			if (!dump(stdout, NULL)) {
				perror("stdout");
				exit(1);
			}
		}
    break;

  case 44:

/* Line 1806 of yacc.c  */
#line 684 "fpd.y"
    {
			exit(0);
		}
    break;

  case 45:

/* Line 1806 of yacc.c  */
#line 688 "fpd.y"
    {
			tsort = begin_tsort();
		}
    break;

  case 46:

/* Line 1806 of yacc.c  */
#line 692 "fpd.y"
    {
			void **sort, **walk;

			sort = end_tsort(tsort);
			for (walk = sort; *walk; walk++)
				printf("%s\n", (char *) *walk);
			free(sort);
		}
    break;

  case 48:

/* Line 1806 of yacc.c  */
#line 704 "fpd.y"
    {
			add_node(tsort, (void *) (yyvsp[(3) - (3)].id), 0);
		}
    break;

  case 49:

/* Line 1806 of yacc.c  */
#line 708 "fpd.y"
    {
			add_node(tsort, (void *) (yyvsp[(3) - (3)].id), 1);
		}
    break;

  case 50:

/* Line 1806 of yacc.c  */
#line 712 "fpd.y"
    {
			struct node *a, *b;

			/* order is important here ! */
			a = add_node(tsort, (void *) (yyvsp[(2) - (4)].id), 0);
			b = add_node(tsort, (void *) (yyvsp[(3) - (4)].id), 0);
			add_edge(a, b, (yyvsp[(4) - (4)].num).n);
		}
    break;

  case 51:

/* Line 1806 of yacc.c  */
#line 724 "fpd.y"
    {
			(yyval.table) = zalloc_type(struct table);
			*next_table = (yyval.table);
			curr_table = (yyval.table);
			n_vars = 0;
		}
    break;

  case 52:

/* Line 1806 of yacc.c  */
#line 731 "fpd.y"
    {
			(yyval.table) = (yyvsp[(2) - (6)].table);
			(yyval.table)->vars = (yyvsp[(4) - (6)].var);
			(yyval.table)->rows = (yyvsp[(6) - (6)].row);
			(yyval.table)->active_row = (yyvsp[(6) - (6)].row);
			next_table = &(yyval.table)->next;
		}
    break;

  case 53:

/* Line 1806 of yacc.c  */
#line 742 "fpd.y"
    {
			(yyval.var) = (yyvsp[(1) - (1)].var);
		}
    break;

  case 54:

/* Line 1806 of yacc.c  */
#line 746 "fpd.y"
    {
			struct var **walk;

			(yyval.var) = (yyvsp[(1) - (3)].var);
			for (walk = &(yyval.var); *walk; walk = &(*walk)->next);
			*walk = (yyvsp[(3) - (3)].var);
		}
    break;

  case 55:

/* Line 1806 of yacc.c  */
#line 757 "fpd.y"
    {
			if (find_var(curr_frame, (yyvsp[(1) - (1)].id))) {
				yyerrorf("duplicate variable \"%s\"", (yyvsp[(1) - (1)].id));
				YYABORT;
			}
			(yyval.var) = zalloc_type(struct var);
			(yyval.var)->name = (yyvsp[(1) - (1)].id);
			(yyval.var)->frame = curr_frame;
			(yyval.var)->table = curr_table;
			(yyval.var)->next = NULL;
			n_vars++;
		}
    break;

  case 56:

/* Line 1806 of yacc.c  */
#line 773 "fpd.y"
    {
			(yyval.row) = NULL;
		}
    break;

  case 57:

/* Line 1806 of yacc.c  */
#line 777 "fpd.y"
    {
			(yyval.row) = alloc_type(struct row);
			(yyval.row)->table = curr_table;
			curr_row = (yyval.row);;
			n_values = 0;
		}
    break;

  case 58:

/* Line 1806 of yacc.c  */
#line 784 "fpd.y"
    {
			if (n_vars != n_values) {
				yyerrorf("table has %d variables but row has "
				    "%d values", n_vars, n_values);
				YYABORT;
			}
			(yyvsp[(2) - (4)].row)->values = (yyvsp[(3) - (4)].value);
		}
    break;

  case 59:

/* Line 1806 of yacc.c  */
#line 793 "fpd.y"
    {
			(yyval.row) = (yyvsp[(2) - (6)].row);
			(yyval.row)->next = (yyvsp[(6) - (6)].row);
		}
    break;

  case 60:

/* Line 1806 of yacc.c  */
#line 801 "fpd.y"
    {
			(yyval.value) = (yyvsp[(1) - (1)].value);
		}
    break;

  case 61:

/* Line 1806 of yacc.c  */
#line 805 "fpd.y"
    {
			struct value **walk;

			(yyval.value) = (yyvsp[(1) - (3)].value);
			for (walk = &(yyval.value); *walk; walk = &(*walk)->next);
			*walk = (yyvsp[(3) - (3)].value);
		}
    break;

  case 62:

/* Line 1806 of yacc.c  */
#line 816 "fpd.y"
    {
			(yyval.value) = alloc_type(struct value);
			(yyval.value)->expr = (yyvsp[(1) - (1)].expr);
			(yyval.value)->row = curr_row;
			(yyval.value)->next = NULL;
			n_values++;
		}
    break;

  case 63:

/* Line 1806 of yacc.c  */
#line 827 "fpd.y"
    {
			(yyval.vec) = alloc_type(struct vec);
			(yyval.vec)->name = NULL;
			(yyval.vec)->base = (yyvsp[(2) - (7)].vec);
			(yyval.vec)->x = (yyvsp[(4) - (7)].expr);
			(yyval.vec)->y = (yyvsp[(6) - (7)].expr);
			(yyval.vec)->frame = curr_frame;
			(yyval.vec)->next = NULL;
			last_vec = (yyval.vec);
			*next_vec = (yyval.vec);
			next_vec = &(yyval.vec)->next;
		}
    break;

  case 64:

/* Line 1806 of yacc.c  */
#line 843 "fpd.y"
    {
			(yyval.vec) = NULL;
		}
    break;

  case 65:

/* Line 1806 of yacc.c  */
#line 847 "fpd.y"
    {
			(yyval.vec) = last_vec;
			if (!(yyval.vec)) {
				yyerrorf(". without predecessor");
				YYABORT;
			}
		}
    break;

  case 66:

/* Line 1806 of yacc.c  */
#line 855 "fpd.y"
    {
			(yyval.vec) = find_vec(curr_frame, (yyvsp[(1) - (1)].id));
			if (!(yyval.vec)) {
				yyerrorf("unknown vector \"%s\"", (yyvsp[(1) - (1)].id));
				YYABORT;
			}
		}
    break;

  case 67:

/* Line 1806 of yacc.c  */
#line 866 "fpd.y"
    {
			(yyval.qvec).frame = curr_frame;
			(yyval.qvec).vec = (yyvsp[(1) - (1)].vec);
		}
    break;

  case 68:

/* Line 1806 of yacc.c  */
#line 871 "fpd.y"
    {
			(yyval.qvec).frame = (yyvsp[(1) - (2)].frame);
			(yyval.qvec).vec = NULL;
		}
    break;

  case 69:

/* Line 1806 of yacc.c  */
#line 876 "fpd.y"
    {
			(yyval.qvec).frame = (yyvsp[(1) - (2)].frame);
			(yyval.qvec).vec = find_vec((yyvsp[(1) - (2)].frame), (yyvsp[(2) - (2)].id));
			if (!(yyval.qvec).vec) {
				yyerrorf("unknown vector \"%s.%s\"",
				    (yyvsp[(1) - (2)].frame)->name, (yyvsp[(2) - (2)].id));
				YYABORT;
			}
		}
    break;

  case 70:

/* Line 1806 of yacc.c  */
#line 889 "fpd.y"
    {
			(yyval.frame) = find_frame((yyvsp[(1) - (2)].id));
			if (!(yyval.frame)) {
				yyerrorf("unknown frame \"%s\"", (yyvsp[(1) - (2)].id));
				YYABORT;
			}
		}
    break;

  case 71:

/* Line 1806 of yacc.c  */
#line 900 "fpd.y"
    {
			(yyval.obj) = (yyvsp[(1) - (1)].obj);
			*next_obj = (yyvsp[(1) - (1)].obj);
			next_obj = &(yyvsp[(1) - (1)].obj)->next;
		}
    break;

  case 72:

/* Line 1806 of yacc.c  */
#line 909 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_pad);
			(yyval.obj)->base = (yyvsp[(3) - (5)].vec);
			(yyval.obj)->u.pad.name = (yyvsp[(2) - (5)].str);
			(yyval.obj)->u.pad.other = (yyvsp[(4) - (5)].vec);
			(yyval.obj)->u.pad.rounded = 0;
			(yyval.obj)->u.pad.type = (yyvsp[(5) - (5)].pt);
		}
    break;

  case 73:

/* Line 1806 of yacc.c  */
#line 918 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_pad);
			(yyval.obj)->base = (yyvsp[(3) - (5)].vec);
			(yyval.obj)->u.pad.name = (yyvsp[(2) - (5)].str);
			(yyval.obj)->u.pad.other = (yyvsp[(4) - (5)].vec);
			(yyval.obj)->u.pad.rounded = 1;
			(yyval.obj)->u.pad.type = (yyvsp[(5) - (5)].pt);
		}
    break;

  case 74:

/* Line 1806 of yacc.c  */
#line 927 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_hole);
			(yyval.obj)->base = (yyvsp[(2) - (3)].vec);
			(yyval.obj)->u.hole.other = (yyvsp[(3) - (3)].vec);
		}
    break;

  case 75:

/* Line 1806 of yacc.c  */
#line 933 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_rect);
			(yyval.obj)->base = (yyvsp[(2) - (4)].vec);
			(yyval.obj)->u.rect.other = (yyvsp[(3) - (4)].vec);
			(yyval.obj)->u.rect.width = (yyvsp[(4) - (4)].expr);
		}
    break;

  case 76:

/* Line 1806 of yacc.c  */
#line 940 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_line);
			(yyval.obj)->base = (yyvsp[(2) - (4)].vec);
			(yyval.obj)->u.line.other = (yyvsp[(3) - (4)].vec);
			(yyval.obj)->u.line.width = (yyvsp[(4) - (4)].expr);
		}
    break;

  case 77:

/* Line 1806 of yacc.c  */
#line 947 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_arc);
			(yyval.obj)->base = (yyvsp[(2) - (4)].vec);
			(yyval.obj)->u.arc.start = (yyvsp[(3) - (4)].vec);
			(yyval.obj)->u.arc.end = (yyvsp[(3) - (4)].vec);
			(yyval.obj)->u.arc.width = (yyvsp[(4) - (4)].expr);
		}
    break;

  case 78:

/* Line 1806 of yacc.c  */
#line 955 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_arc);
			(yyval.obj)->base = (yyvsp[(2) - (5)].vec);
			(yyval.obj)->u.arc.start = (yyvsp[(3) - (5)].vec);
			(yyval.obj)->u.arc.end = (yyvsp[(4) - (5)].vec);
			(yyval.obj)->u.arc.width = (yyvsp[(5) - (5)].expr);
		}
    break;

  case 79:

/* Line 1806 of yacc.c  */
#line 963 "fpd.y"
    { 
		    (yyval.num).n = lineno;
		}
    break;

  case 80:

/* Line 1806 of yacc.c  */
#line 967 "fpd.y"
    {
			(yyval.obj) = new_obj(ot_frame);
			(yyval.obj)->base = (yyvsp[(4) - (4)].vec);
			(yyval.obj)->u.frame.ref = find_frame((yyvsp[(2) - (4)].id));
			if (!(yyval.obj)->u.frame.ref) {
				yyerrorf("unknown frame \"%s\"", (yyvsp[(2) - (4)].id));
				YYABORT;
			}
			if (!(yyval.obj)->u.frame.ref->active_ref)
				(yyval.obj)->u.frame.ref->active_ref = (yyval.obj);
			(yyval.obj)->u.frame.lineno = (yyvsp[(3) - (4)].num).n;
		}
    break;

  case 81:

/* Line 1806 of yacc.c  */
#line 982 "fpd.y"
    {
			(yyval.pt) = pt_normal;
		}
    break;

  case 82:

/* Line 1806 of yacc.c  */
#line 986 "fpd.y"
    {
			if (!strcmp((yyvsp[(1) - (1)].id), "bare"))
				(yyval.pt) = pt_bare;
			else if (!strcmp((yyvsp[(1) - (1)].id), "trace"))
				(yyval.pt) = pt_trace;
			else if (!strcmp((yyvsp[(1) - (1)].id), "paste"))
				(yyval.pt) = pt_paste;
			else if (!strcmp((yyvsp[(1) - (1)].id), "mask"))
				(yyval.pt) = pt_mask;
			else {
				yyerrorf("unknown pad type \"%s\"", (yyvsp[(1) - (1)].id));
				YYABORT;
			}
		}
    break;

  case 85:

/* Line 1806 of yacc.c  */
#line 1008 "fpd.y"
    {
			*next_obj = (yyvsp[(1) - (1)].obj);
			next_obj = &(yyvsp[(1) - (1)].obj)->next;
		}
    break;

  case 86:

/* Line 1806 of yacc.c  */
#line 1013 "fpd.y"
    {
			*next_obj = (yyvsp[(2) - (2)].obj);
			next_obj = &(yyvsp[(2) - (2)].obj)->next;
		}
    break;

  case 88:

/* Line 1806 of yacc.c  */
#line 1022 "fpd.y"
    {
			(yyval.obj) = (yyvsp[(1) - (1)].obj);
		}
    break;

  case 89:

/* Line 1806 of yacc.c  */
#line 1026 "fpd.y"
    {
			(yyval.obj) = (yyvsp[(2) - (2)].obj);
			if (find_label(curr_frame, (yyvsp[(1) - (2)].id))) {
				yyerrorf("duplicate label \"%s\"", (yyvsp[(1) - (2)].id));
				YYABORT;
			}
			(yyval.obj)->name = (yyvsp[(1) - (2)].id);
		}
    break;

  case 90:

/* Line 1806 of yacc.c  */
#line 1038 "fpd.y"
    {
			struct meas *meas;

			(yyval.obj) = new_obj(ot_meas);
			meas = &(yyval.obj)->u.meas;
			meas->type = (yyvsp[(4) - (6)].mo).max ? (yyvsp[(1) - (6)].mt)+3 : (yyvsp[(1) - (6)].mt);
			meas->label = (yyvsp[(2) - (6)].str);
			(yyval.obj)->base = (yyvsp[(3) - (6)].qbase).vec;
			meas->low_qual = (yyvsp[(3) - (6)].qbase).qualifiers;
			meas->inverted = (yyvsp[(4) - (6)].mo).inverted;
			meas->high = (yyvsp[(5) - (6)].qbase).vec;
			meas->high_qual = (yyvsp[(5) - (6)].qbase).qualifiers;
			meas->offset = (yyvsp[(6) - (6)].expr);
			(yyval.obj)->next = NULL;
		}
    break;

  case 91:

/* Line 1806 of yacc.c  */
#line 1057 "fpd.y"
    {
			(yyval.qbase) = (yyvsp[(1) - (1)].qbase);
			if (!check_qbase(&(yyval.qbase)))
				YYABORT;
		}
    break;

  case 92:

/* Line 1806 of yacc.c  */
#line 1066 "fpd.y"
    {
			(yyval.qbase).vec = find_vec(frames, (yyvsp[(1) - (1)].id));
			if (!(yyval.qbase).vec) {
				yyerrorf("unknown vector \"%s\"", (yyvsp[(1) - (1)].id));
				YYABORT;
			}
			(yyval.qbase).qualifiers = NULL;
		}
    break;

  case 93:

/* Line 1806 of yacc.c  */
#line 1075 "fpd.y"
    {
			const struct frame *frame;

			frame = find_frame((yyvsp[(1) - (3)].id));
			(yyval.qbase).vec = frame ? find_vec(frame, (yyvsp[(3) - (3)].id)) : NULL;
			if (!(yyval.qbase).vec) {
				yyerrorf("unknown vector \"%s.%s\"", (yyvsp[(1) - (3)].id), (yyvsp[(3) - (3)].id));
				YYABORT;
			}
			(yyval.qbase).qualifiers = NULL;
		}
    break;

  case 94:

/* Line 1806 of yacc.c  */
#line 1087 "fpd.y"
    {
			const struct frame *frame;
			struct frame_qual *qual;

			(yyval.qbase) = (yyvsp[(3) - (3)].qbase);
			frame = find_frame((yyvsp[(1) - (3)].id));
			if (!frame) {
				yyerrorf("unknown frame \"%s\"", (yyvsp[(1) - (3)].id));
				YYABORT;
			} else {
				qual = alloc_type(struct frame_qual);
				qual->frame = frame;
				qual->next = (yyval.qbase).qualifiers;
				(yyval.qbase).qualifiers = qual;
			}
		}
    break;

  case 95:

/* Line 1806 of yacc.c  */
#line 1107 "fpd.y"
    {
			(yyval.mt) = mt_xy_next;
		}
    break;

  case 96:

/* Line 1806 of yacc.c  */
#line 1111 "fpd.y"
    {
			(yyval.mt) = mt_x_next;
		}
    break;

  case 97:

/* Line 1806 of yacc.c  */
#line 1115 "fpd.y"
    {
			(yyval.mt) = mt_y_next;
		}
    break;

  case 98:

/* Line 1806 of yacc.c  */
#line 1122 "fpd.y"
    {
			(yyval.mo).max = 0;
			(yyval.mo).inverted = 0;
		}
    break;

  case 99:

/* Line 1806 of yacc.c  */
#line 1127 "fpd.y"
    {
			(yyval.mo).max = 0;
			(yyval.mo).inverted = 1;
		}
    break;

  case 100:

/* Line 1806 of yacc.c  */
#line 1132 "fpd.y"
    {
			(yyval.mo).max = 1;
			(yyval.mo).inverted = 0;
		}
    break;

  case 101:

/* Line 1806 of yacc.c  */
#line 1137 "fpd.y"
    {
			(yyval.mo).max = 1;
			(yyval.mo).inverted = 1;
		}
    break;

  case 102:

/* Line 1806 of yacc.c  */
#line 1144 "fpd.y"
    {
			(yyval.num).n = 0;
		}
    break;

  case 103:

/* Line 1806 of yacc.c  */
#line 1148 "fpd.y"
    {
			(yyval.num) = (yyvsp[(1) - (1)].num);
		}
    break;

  case 104:

/* Line 1806 of yacc.c  */
#line 1154 "fpd.y"
    {
			(yyval.str) = NULL;
		}
    break;

  case 105:

/* Line 1806 of yacc.c  */
#line 1158 "fpd.y"
    {
			(yyval.str) = (yyvsp[(1) - (1)].str);
		}
    break;

  case 106:

/* Line 1806 of yacc.c  */
#line 1164 "fpd.y"
    {
			(yyval.expr) = NULL;
		}
    break;

  case 107:

/* Line 1806 of yacc.c  */
#line 1168 "fpd.y"
    {
			(yyval.expr) = (yyvsp[(1) - (1)].expr);
		}
    break;

  case 108:

/* Line 1806 of yacc.c  */
#line 1175 "fpd.y"
    {
			(yyval.expr) = (yyvsp[(1) - (1)].expr);
		}
    break;

  case 109:

/* Line 1806 of yacc.c  */
#line 1182 "fpd.y"
    {
			(yyval.expr) = (yyvsp[(1) - (1)].expr);
		}
    break;

  case 110:

/* Line 1806 of yacc.c  */
#line 1186 "fpd.y"
    {
			(yyval.expr) = binary_op(op_add, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
		}
    break;

  case 111:

/* Line 1806 of yacc.c  */
#line 1190 "fpd.y"
    {
			(yyval.expr) = binary_op(op_sub, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
		}
    break;

  case 112:

/* Line 1806 of yacc.c  */
#line 1197 "fpd.y"
    {
			(yyval.expr) = (yyvsp[(1) - (1)].expr);
		}
    break;

  case 113:

/* Line 1806 of yacc.c  */
#line 1201 "fpd.y"
    {
			(yyval.expr) = binary_op(op_mult, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
		}
    break;

  case 114:

/* Line 1806 of yacc.c  */
#line 1205 "fpd.y"
    {
			(yyval.expr) = binary_op(op_div, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
		}
    break;

  case 115:

/* Line 1806 of yacc.c  */
#line 1212 "fpd.y"
    {
			(yyval.expr) = (yyvsp[(1) - (1)].expr);
		}
    break;

  case 116:

/* Line 1806 of yacc.c  */
#line 1216 "fpd.y"
    {
			(yyval.expr) = binary_op(op_minus, (yyvsp[(2) - (2)].expr), NULL);
		}
    break;

  case 117:

/* Line 1806 of yacc.c  */
#line 1223 "fpd.y"
    {
			(yyval.expr) = new_op(op_num);
			(yyval.expr)->u.num = (yyvsp[(1) - (1)].num);
		}
    break;

  case 118:

/* Line 1806 of yacc.c  */
#line 1228 "fpd.y"
    {
			(yyval.expr) = new_op(op_var);
			(yyval.expr)->u.var = (yyvsp[(1) - (1)].id);
		}
    break;

  case 119:

/* Line 1806 of yacc.c  */
#line 1233 "fpd.y"
    {
			(yyval.expr) = new_op(op_string);
			(yyval.expr)->u.str = (yyvsp[(1) - (1)].str);
		}
    break;

  case 120:

/* Line 1806 of yacc.c  */
#line 1238 "fpd.y"
    {
			(yyval.expr) = (yyvsp[(2) - (3)].expr);
		}
    break;

  case 121:

/* Line 1806 of yacc.c  */
#line 1242 "fpd.y"
    {
			if ((yyvsp[(1) - (4)].id) == id_sin)
				(yyval.expr) = binary_op(op_sin, (yyvsp[(3) - (4)].expr), NULL);
			else if ((yyvsp[(1) - (4)].id) == id_cos)
				(yyval.expr) = binary_op(op_cos, (yyvsp[(3) - (4)].expr), NULL);
			else if ((yyvsp[(1) - (4)].id) == id_sqrt)
				(yyval.expr) = binary_op(op_sqrt, (yyvsp[(3) - (4)].expr), NULL);
			else {
				yyerrorf("unknown function \"%s\"", (yyvsp[(1) - (4)].id));
				YYABORT;
			}
		}
    break;

  case 122:

/* Line 1806 of yacc.c  */
#line 1259 "fpd.y"
    {
			(yyval.value) = NULL;
		}
    break;

  case 123:

/* Line 1806 of yacc.c  */
#line 1263 "fpd.y"
    {
			(yyval.value) = (yyvsp[(2) - (2)].value);
		}
    break;



/* Line 1806 of yacc.c  */
#line 3211 "y.tab.c"
      default: break;
    }
  /* User semantic actions sometimes alter yychar, and that requires
     that yytoken be updated with the new translation.  We take the
     approach of translating immediately before every use of yytoken.
     One alternative is translating here after every semantic action,
     but that translation would be missed if the semantic action invokes
     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
     incorrect destructor might then be invoked immediately.  In the
     case of YYERROR or YYBACKUP, subsequent parser actions might lead
     to an incorrect destructor call or verbose syntax error message
     before the lookahead is translated.  */
  YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);

  *++yyvsp = yyval;

  /* Now `shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTOKENS] + *yyssp;
  if (0 <= yystate && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTOKENS];

  goto yynewstate;


/*------------------------------------.
| yyerrlab -- here on detecting error |
`------------------------------------*/
yyerrlab:
  /* Make sure we have latest lookahead translation.  See comments at
     user semantic actions for why this is necessary.  */
  yytoken = yychar == YYEMPTY ? YYEMPTY : YYTRANSLATE (yychar);

  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
#if ! YYERROR_VERBOSE
      yyerror (YY_("syntax error"));
#else
# define YYSYNTAX_ERROR yysyntax_error (&yymsg_alloc, &yymsg, \
                                        yyssp, yytoken)
      {
        char const *yymsgp = YY_("syntax error");
        int yysyntax_error_status;
        yysyntax_error_status = YYSYNTAX_ERROR;
        if (yysyntax_error_status == 0)
          yymsgp = yymsg;
        else if (yysyntax_error_status == 1)
          {
            if (yymsg != yymsgbuf)
              YYSTACK_FREE (yymsg);
            yymsg = (char *) YYSTACK_ALLOC (yymsg_alloc);
            if (!yymsg)
              {
                yymsg = yymsgbuf;
                yymsg_alloc = sizeof yymsgbuf;
                yysyntax_error_status = 2;
              }
            else
              {
                yysyntax_error_status = YYSYNTAX_ERROR;
                yymsgp = yymsg;
              }
          }
        yyerror (yymsgp);
        if (yysyntax_error_status == 2)
          goto yyexhaustedlab;
      }
# undef YYSYNTAX_ERROR
#endif
    }



  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
	 error, discard it.  */

      if (yychar <= YYEOF)
	{
	  /* Return failure if at end of input.  */
	  if (yychar == YYEOF)
	    YYABORT;
	}
      else
	{
	  yydestruct ("Error: discarding",
		      yytoken, &yylval);
	  yychar = YYEMPTY;
	}
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:

  /* Pacify compilers like GCC when the user code never invokes
     YYERROR and the label yyerrorlab therefore never appears in user
     code.  */
  if (/*CONSTCOND*/ 0)
     goto yyerrorlab;

  /* Do not reclaim the symbols of the rule which action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;	/* Each real token shifted decrements this.  */

  for (;;)
    {
      yyn = yypact[yystate];
      if (!yypact_value_is_default (yyn))
	{
	  yyn += YYTERROR;
	  if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
	    {
	      yyn = yytable[yyn];
	      if (0 < yyn)
		break;
	    }
	}

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
	YYABORT;


      yydestruct ("Error: popping",
		  yystos[yystate], yyvsp);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  *++yyvsp = yylval;


  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;

/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;

#if !defined(yyoverflow) || YYERROR_VERBOSE
/*-------------------------------------------------.
| yyexhaustedlab -- memory exhaustion comes here.  |
`-------------------------------------------------*/
yyexhaustedlab:
  yyerror (YY_("memory exhausted"));
  yyresult = 2;
  /* Fall through.  */
#endif

yyreturn:
  if (yychar != YYEMPTY)
    {
      /* Make sure we have latest lookahead translation.  See comments at
         user semantic actions for why this is necessary.  */
      yytoken = YYTRANSLATE (yychar);
      yydestruct ("Cleanup: discarding lookahead",
                  yytoken, &yylval);
    }
  /* Do not reclaim the symbols of the rule which action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
		  yystos[*yyssp], yyvsp);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
#if YYERROR_VERBOSE
  if (yymsg != yymsgbuf)
    YYSTACK_FREE (yymsg);
#endif
  /* Make sure YYID is used.  */
  return YYID (yyresult);
}



